/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.clientlib.parse;

import cricketdaemon.Util;
import cricketdaemon.clientlib.data.*;
import java.io.*;
import java.util.*;

/**
 * Parses subtypes contained within Distance Estimate Array.
 *
 * @author Allen Miu
 */
class SampleArrayParseHandlerMap extends ParseHandlerMap
{
    public static SampleArrayParseHandlerMap instance = 
	new SampleArrayParseHandlerMap();

    public static SampleParseHandlerMap sampleParser = 
	    new SampleParseHandlerMap();

    protected SampleArrayParseHandlerMap()
    {
	super();
	register("sample");
    }

    private static class SampleParseHandlerMap extends ParseHandlerMap
    {
	/*
	public static SampleParseHandlerMap instance = 
	    new SampleParseHandlerMap();
	*/

	SampleParseHandlerMap()
	{
	    super();
	    register("dist", "check_sample", "parse_dist");
	    register("time", "check_sample", "parse_time");
	}

	public boolean check_sample(Object o)
	{
	    return o instanceof Sample;
	}

	public void parse_dist(Object dataStructure, String value)
	    throws IOException 
	{
	    Sample s = (Sample) dataStructure;
	    s.dist = Util.parseDouble(value);
	}

	public void parse_time(Object dataStructure, String value)
	    throws IOException
	{
	    Sample s = (Sample) dataStructure;
	    s.time = Long.parseLong(value);
	}
    }

    public boolean check_sample(Object o)
    {
	return o instanceof ArrayList;
    }

    public void parse_sample(Object dataStructure, String value)
	throws IOException 
    {
	Sample s = new Sample();
	ArrayList a = (ArrayList) dataStructure;
	a.add(s);
	
	StringReader sr = new StringReader(value);
	while(Field.newField(sr)) { // new dest_est record
	    Field.parseField(s, sr, sampleParser);
	}
    }
}




