/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader;

import cricketdaemon.CricketDaemon;
import cricketdaemon.Util;
import cricketdaemon.module.*;
import cricketdaemon.beacontable.*;
import cricketdaemon.reader.parse.*;
import java.io.*;
import java.net.*;
import java.util.*;

/**
 * Retrieves and parses beacon data from the gpsd.
 *
 * @author Allen Miu
 */
public class GPSDReaderV2 extends GPSDReader
{
    public GPSDReaderV2(String addr, int port)
    {
	super(addr, port);
	MAGIC_TOKEN = "VR=2.0";
    }

    long lastHW;
    long lastClock;

    protected void transfer(Data data, BitSet mask)
    {
	Beacon beacon = (Beacon) data;
	insertVal(beacon, mask, MAGIC_TOKEN);
	insertVal(beacon, mask, "id=0");
	if(!mask.get(ParseMap.SPACE)) {
	    beacon.insertInv(beacon.getField(ParseMap.UNIQUEID), 
			     ParseMap.SPACE);
	    mask.set(ParseMap.SPACE);
	}

	//System.out.println("GPSD cpu elapse="+(beacon.timeStamp.longValue()-lastClock)+" "+
	//"hw elapse="+(beacon.hwTimeStamp.longValue()-lastHW));
	lastHW = beacon.hwTimeStamp.longValue();
	lastClock = beacon.timeStamp.longValue();

	super.transfer(beacon, mask);
    }

    protected void moduleCleanUp()
    {  // never really gets called because run() is overridden
	return;
    }

    public static void main(String[] args) 
    {
	try {
	    CricketDaemon.rawGPSDPrint = true;

	    GPSDReader g = 
		new GPSDReader("18.24.6.238", 2947);
	    DebugModule d = new DebugModule(g);

	    /* comment out the above and uncomment this for translation
	       GPSDReader g = 
	       new GPSDReader(InetAddress.getByName("18.24.6.238"), 2947);
	       BeaconTableModule b = new BeaconTableModule(g, "translate.conf");
	       DebugModule d = new DebugModule(b);
	       b.start();
	    */ 

	    g.start();
	    d.start();
	    g.join();
	}
	catch (InterruptedException e) {
	}
    }
}

